import { createToken, createPlugin } from 'fusion-core';
import { UniversalEventsToken } from 'fusion-plugin-universal-events';
import { I18nToken } from 'fusion-plugin-i18n';
import { FetchToken } from 'fusion-tokens';
import mapObject from 'just-map-object';
import isEqual from 'just-compare';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var RPCToken = createToken('RPCToken');
var RPCHandlersToken = createToken('RPCHandlersToken');
var BodyParserOptionsToken = createToken('BodyParserOptionsToken');
var RPCHandlersConfigToken = createToken('RPCHandlersConfigToken');

var formatApiPath = function formatApiPath(apiPath) {
  return ("/" + apiPath + "/").replace(/\/{2,}/g, '/');
};

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env browser */
var statKey = 'rpc:method-client';

var RPC =
/*#__PURE__*/
function () {
  function RPC(_ref) {
    var fetch = _ref.fetch,
        emitter = _ref.emitter,
        rpcConfig = _ref.rpcConfig,
        localeCode = _ref.localeCode;
    this.fetch = fetch;
    this.config = rpcConfig || {};
    this.emitter = emitter;
    this.localeCode = localeCode;
    this.apiPath = formatApiPath(rpcConfig && rpcConfig.apiPath ? rpcConfig.apiPath : 'api');
  }

  var _proto = RPC.prototype;

  _proto.request = function request(rpcId, args, headers) {
    if (!this.fetch) {
      throw new Error('fusion-plugin-rpc requires `fetch`');
    }

    if (!this.emitter) {
      throw new Error('Missing emitter registered to UniversalEventsToken');
    }

    var fetch = this.fetch;
    var emitter = this.emitter;
    var apiPath = this.apiPath;
    var startTime = Date.now();
    var localeParam = this.localeCode ? "?localeCode=" + this.localeCode : '';
    return fetch("" + apiPath + rpcId + localeParam, args instanceof FormData ? {
      method: 'POST',
      headers: _objectSpread({}, headers || {}),
      body: args
    } : {
      method: 'POST',
      headers: _objectSpread({
        'Content-Type': 'application/json'
      }, headers || {}),
      body: JSON.stringify(args || {})
    }).then(function (r) {
      return r.json();
    }).then(function (args) {
      var status = args.status,
          data = args.data;

      if (status === 'success') {
        emitter.emit(statKey, {
          method: rpcId,
          status: 'success',
          timing: Date.now() - startTime
        });
        return data;
      } else {
        emitter.emit(statKey, {
          method: rpcId,
          error: data,
          status: 'failure',
          timing: Date.now() - startTime
        });
        return Promise.reject(data);
      }
    });
  };

  return RPC;
}();

var pluginFactory = function pluginFactory() {
  return createPlugin({
    deps: {
      fetch: FetchToken,
      emitter: UniversalEventsToken,
      i18n: I18nToken.optional,
      rpcConfig: RPCHandlersConfigToken.optional
    },
    provides: function provides(deps) {
      var _deps$fetch = deps.fetch,
          fetch = _deps$fetch === void 0 ? window.fetch : _deps$fetch,
          emitter = deps.emitter,
          rpcConfig = deps.rpcConfig,
          i18n = deps.i18n;
      return {
        from: function from(ctx) {
          var locale = i18n && i18n.from(ctx).locale || '';
          var localeCode = typeof locale === 'string' ? locale : locale.code;
          return new RPC({
            fetch: fetch,
            emitter: emitter,
            rpcConfig: rpcConfig,
            localeCode: localeCode
          });
        }
      };
    }
  });
};

var browserDataFetching = true && pluginFactory();

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var MissingHandlerError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(MissingHandlerError, _Error);

  function MissingHandlerError(method) {
    var _this;

    _this = _Error.call(this, "Missing RPC handler for " + method) || this;
    _this.code = 'ERR_MISSING_HANDLER';
    Error.captureStackTrace(_this, MissingHandlerError);
    return _this;
  }

  return MissingHandlerError;
}(Error);

function _inheritsLoose$1(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var ResponseError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose$1(ResponseError, _Error);

  function ResponseError(message) {
    var _this;

    _this = _Error.call(this, message) || this;
    _this.code = null;
    _this.meta = null;
    Error.captureStackTrace(_this, ResponseError);
    return _this;
  }

  return ResponseError;
}(Error);

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env node */

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var RPC$2 =
/*#__PURE__*/
function () {
  function RPC(handlers) {
    this.handlers = handlers;
  }

  var _proto = RPC.prototype;

  _proto.request = function request(method, args) {
    return new Promise(function ($return, $error) {
      if (!this.handlers) {
        return $error(new Error('fusion-plugin-rpc requires `handlers`'));
      }

      if (!this.handlers[method]) {
        return $error(new MissingHandlerError(method));
      }

      return $return(this.handlers[method](args));
    }.bind(this));
  };

  return RPC;
}();

var plugin = createPlugin({
  deps: {
    handlers: RPCHandlersToken,
    // $FlowFixMe
    emitter: UniversalEventsToken.optional
  },
  provides: function provides(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        handlers = _ref.handlers;

    return {
      from: function from() {
        return new RPC$2(handlers);
      }
    };
  }
});

function ownKeys$1(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread$1(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys$1(source, true).forEach(function (key) { _defineProperty$1(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys$1(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty$1(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var getMockRpcHandlers = function getMockRpcHandlers(fixtures, onMockRpc) {
  return fixtures.reduce(function (rpcHandlers, fixture) {
    return _objectSpread$1({}, rpcHandlers, {}, mapObject(fixture, function (rpcId, responseDetails) {
      return function () {
        var $args = arguments;
        return new Promise(function ($return, $error) {
          for (var _len = $args.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = $args[_key];
          }

          var response = Array.isArray(responseDetails) ? responseDetails.filter(function (item) {
            return isEqual(item.args, args);
          })[0].response : responseDetails;
          onMockRpc && onMockRpc(rpcId, args, response);

          if (response instanceof Error) {
            return $error(response);
          }

          return $return(response);
        });
      };
    }));
  }, {});
};

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var index = browserDataFetching;

export default index;
export { plugin as mock, ResponseError, BodyParserOptionsToken, RPCToken, RPCHandlersToken, RPCHandlersConfigToken, getMockRpcHandlers };
